//
// Block structured mesh for 2D lid driven cavity flow
//
//////////////////////////////////////////////////////////////////////
// Parameter section
//////////////////////////////////////////////////////////////////////
// I use open cascade
SetFactory("OpenCASCADE");

// mesh parameters
// number of splits
nsplit = DefineNumber[ 4 , Min 2, Max 50, Step 1,   Name "Numer of splits" ];
// progression
prog = DefineNumber[ 1.2 , Min 1, Max 2, Step 0.1,   Name "Progression" ];

// mesh parameters
box = 0.5;

// Element scale at cylinder surface
cs_el_sc = 0.1;

//////////////////////////////////////////////////////////////////////
// Square region around cylinder
//////////////////////////////////////////////////////////////////////
// Points
pts_list = {};

pts_list += {newp};
Point(newp) = {-box,0.0,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {0.0,0.0,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {box,0.0,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {-box,-box,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {0.0,-box,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {box,-box,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {-box,-2.0*box,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {0.0,-2.0*box,0.0,cs_el_sc};
pts_list += {newp};
Point(newp) = {box,-2.0*box,0.0,cs_el_sc};

// Lines
line_list = {};
// horizontal
line_list += {newl};
Line(newl) = {pts_list[0],pts_list[1]};
line_list += {newl};
Line(newl) = {pts_list[1],pts_list[2]};
line_list += {newl};
Line(newl) = {pts_list[3],pts_list[4]};
line_list += {newl};
Line(newl) = {pts_list[4],pts_list[5]};
line_list += {newl};
Line(newl) = {pts_list[6],pts_list[7]};
line_list += {newl};
Line(newl) = {pts_list[7],pts_list[8]};
// vertical
line_list += {newl};
Line(newl) = {pts_list[3],pts_list[0]};
line_list += {newl};
Line(newl) = {pts_list[4],pts_list[1]};
line_list += {newl};
Line(newl) = {pts_list[5],pts_list[2]};
line_list += {newl};
Line(newl) = {pts_list[6],pts_list[3]};
line_list += {newl};
Line(newl) = {pts_list[7],pts_list[4]};
line_list += {newl};
Line(newl) = {pts_list[8],pts_list[5]};

// Surfaces
surface_list = {};

crvl = newll;
Curve Loop (crvl) = {line_list[2],line_list[7],-line_list[0],-line_list[6]};
surface_list += {news};
Surface(news) = {crvl};

crvl = newll;
Curve Loop (crvl) = {line_list[3],line_list[8],-line_list[1],-line_list[7]};
surface_list += {news};
Surface(news) = {crvl};

crvl = newll;
Curve Loop (crvl) = {line_list[4],line_list[10],-line_list[2],-line_list[9]};
surface_list += {news};
Surface(news) = {crvl};

crvl = newll;
Curve Loop (crvl) = {line_list[5],line_list[11],-line_list[3],-line_list[10]};
surface_list += {news};
Surface(news) = {crvl};

//////////////////////////////////////////////////////////////////////
// Physical properties section
//////////////////////////////////////////////////////////////////////
// Volume
Physical Surface("Fluid",1) = {surface_list[]};

// Top
Physical Curve("Dir",2) = {line_list[0],line_list[1]};

// Wall
Physical Curve("Wal",3) = {line_list[4],line_list[5],line_list[6],line_list[8],line_list[9],line_list[11]};

//////////////////////////////////////////////////////////////////////
// Transfinite division section
//////////////////////////////////////////////////////////////////////
// Edge division
//
Transfinite Curve{line_list[0],-line_list[1],line_list[2],-line_list[3],line_list[4],-line_list[5],-line_list[6],-line_list[7],-line_list[8],line_list[9],line_list[10],line_list[11]} = (nsplit+1) Using Progression prog;

// Surface division
Transfinite Surface{surface_list[]};
Recombine Surface{surface_list[]};


//////////////////////////////////////////////////////////////////////
// Meshing section
//////////////////////////////////////////////////////////////////////
Mesh 1;
Mesh 2;
Mesh 3;

SetOrder 2;

RenumberMeshElements;

//////////////////////////////////////////////////////////////////////
// Mesh saving section
//////////////////////////////////////////////////////////////////////
Mesh.Format = 1;
Mesh.MshFileVersion = 2.2;
Mesh.SaveAll = 0;
Mesh.Binary = 0;

Save "lid_dcav.msh";

